@@ -91,8 +91,7 @@ def dashboardAll(db):
         results.append(get_count_by_perc(exe_perc_data, 89, 80))
         results.append(get_count_by_perc(exe_perc_data, 79, 70))
         results.append(get_count_by_perc(exe_perc_data, 69, 60))
-        results.append(get_count_by_perc(exe_perc_data, 59, 50))
-        results.append(get_count_by_perc(exe_perc_data, 49, 0))
+        results.append(get_count_by_perc(exe_perc_data, 59, 0))
 
         return render_template('dashboardAll.html', exe_id_avg_data=exe_id_avg_data,
          results=results, results_data=results_data, db_name=db)
@@ -117,6 +116,11 @@ def dashboardRecent(db):
         cursor.execute("SELECT Execution_Id, Execution_Total from TB_EXECUTION order by Execution_Id desc LIMIT 2;")
         exe_info = cursor.fetchall()
 
+        if len(exe_info) == 2:
+            pass
+        else:
+            exe_info = (exe_info[0], exe_info[0])
+
         cursor.execute("SELECT Execution_Pass, Execution_Fail, Execution_Total, Execution_Time from TB_EXECUTION WHERE Execution_Id=%s;" % exe_info[0][0])
         last_exe_data = cursor.fetchall()
 
@@ -132,7 +136,7 @@ def dashboardRecent(db):
         cursor.execute("SELECT ROUND(AVG(Test_Time),2) from TB_TEST WHERE Execution_Id=%s;" % exe_info[0][0])
         test_avg_dur_data = cursor.fetchall()
 
-        cursor.execute("SELECT Suite_Name, Suite_Fail, COUNT(Suite_Name) as F from TB_SUITE WHERE Suite_Status='FAIL' AND Execution_Id >= %s GROUP BY Suite_Name HAVING COUNT(Suite_Name) > 1 ORDER BY F DESC, Suite_Fail DESC LIMIT 5;" % exe_info[1][0])
+        cursor.execute("SELECT b.Suite_Name, a.Suite_Fail, b.Occurence from TB_SUITE a INNER JOIN (Select Suite_Name, Count(Suite_Name) as Occurence From TB_SUITE WHERE Suite_Status = 'FAIL' AND Execution_Id>=%s GROUP BY Suite_Name HAVING COUNT(Suite_Name) > 1) b ON a.Suite_Name = b.Suite_Name WHERE Suite_Status='FAIL' AND Execution_Id=%s ORDER BY b.Occurence DESC, a.Suite_Fail DESC LIMIT 5;" % (exe_info[-1][0], exe_info[0][0]))
         common_failed_suites = cursor.fetchall()
     
         cursor.execute("SELECT COUNT(*) From (SELECT Test_Name, Execution_Id From TB_TEST WHERE Test_Status='FAIL' AND Execution_Id >= %s GROUP BY Test_Name HAVING COUNT(Test_Name) = 1) AS T WHERE Execution_Id=%s" % (exe_info[1][0],exe_info[0][0]))
@@ -169,6 +173,80 @@ def dashboardRecent(db):
     else:
         return redirect(url_for('redirect_url'))
 
+@app.route('/<db>/dashboard/<eid>', methods=['GET'])
+def eid_dashboard(db, eid):
+    cursor = mysql.connection.cursor()
+    use_db(cursor, db)
+
+    cursor.execute("SELECT COUNT(Execution_Id) from TB_EXECUTION;")
+    results_data = cursor.fetchall()
+    cursor.execute("SELECT COUNT(Suite_Id) from TB_SUITE;")
+    suite_results_data = cursor.fetchall()
+    cursor.execute("SELECT COUNT(Test_Id) from TB_TEST;")
+    test_results_data = cursor.fetchall()
+
+    if results_data[0][0] > 0 and suite_results_data[0][0] > 0 and test_results_data[0][0] > 0:
+
+        cursor.execute("SELECT Execution_Id, Execution_Total from TB_EXECUTION WHERE Execution_Id <=%s order by Execution_Id desc LIMIT 2;" % eid)
+        exe_info = cursor.fetchall()
+
+        if len(exe_info) == 2:
+            pass
+        else:
+            exe_info = (exe_info[0], exe_info[0])
+
+        cursor.execute("SELECT Execution_Pass, Execution_Fail, Execution_Total, Execution_Time from TB_EXECUTION WHERE Execution_Id=%s;" % exe_info[0][0])
+        last_exe_data = cursor.fetchall()
+
+        cursor.execute("SELECT Execution_Pass, Execution_Fail, Execution_Total, Execution_Time from TB_EXECUTION WHERE Execution_Id=%s;" % exe_info[1][0])
+        prev_exe_data = cursor.fetchall()
+
+        cursor.execute("SELECT COUNT(*) from TB_TEST WHERE Execution_Id=%s AND Test_Status = 'FAIL' AND Test_Comment IS NULL;" % exe_info[0][0])
+        req_anal_data = cursor.fetchall()
+
+        cursor.execute("SELECT ROUND(AVG(Suite_Time),2) from TB_SUITE WHERE Execution_Id=%s;" % exe_info[0][0])
+        suite_avg_dur_data = cursor.fetchall()
+
+        cursor.execute("SELECT ROUND(AVG(Test_Time),2) from TB_TEST WHERE Execution_Id=%s;" % exe_info[0][0])
+        test_avg_dur_data = cursor.fetchall()
+
+        cursor.execute("SELECT b.Suite_Name, a.Suite_Fail, b.Occurence from TB_SUITE a INNER JOIN (Select Suite_Name, Count(Suite_Name) as Occurence From TB_SUITE WHERE Suite_Status = 'FAIL' AND Execution_Id IN (%s, %s) GROUP BY Suite_Name HAVING COUNT(Suite_Name) > 1) b ON a.Suite_Name = b.Suite_Name WHERE Suite_Status='FAIL' AND Execution_Id=%s ORDER BY b.Occurence DESC, a.Suite_Fail DESC LIMIT 5;" % (exe_info[-1][0], exe_info[0][0], exe_info[0][0]))
+        common_failed_suites = cursor.fetchall()
+    
+        cursor.execute("SELECT COUNT(*) From (SELECT Test_Name, Execution_Id From TB_TEST WHERE Test_Status='FAIL' AND Execution_Id >= %s GROUP BY Test_Name HAVING COUNT(Test_Name) = 1) AS T WHERE Execution_Id=%s" % (exe_info[1][0],exe_info[0][0]))
+        new_failed_tests_count = cursor.fetchall()
+
+        cursor.execute("SELECT COUNT(*) from TB_TEST WHERE Execution_Id=%s AND Test_Comment LIKE '%%Application_Issue%%';" % exe_info[0][0])
+        app_failure_anl_count = cursor.fetchall()
+
+        cursor.execute("SELECT COUNT(*) from TB_TEST WHERE Execution_Id=%s AND Test_Comment LIKE '%%Automation_Issue%%';" % exe_info[0][0])
+        auto_failure_anl_count = cursor.fetchall()
+
+        # required analysis percentage
+        if last_exe_data[0][1] > 0 and last_exe_data[0][1] != req_anal_data[0][0]:
+            req_anal_perc_data = round( ((last_exe_data[0][1] - req_anal_data[0][0]) / last_exe_data[0][1])*100  ,2)
+        else:
+            req_anal_perc_data = 0
+        
+        new_tests_count = exe_info[0][1] - exe_info[1][1]
+        passed_test_dif = last_exe_data[0][0] - prev_exe_data[0][0]
+        failed_test_dif = prev_exe_data[0][1] - last_exe_data[0][1]
+
+        return render_template('dashboardByEid.html', last_exe_data=last_exe_data, exe_info=exe_info,
+         prev_exe_data=prev_exe_data, new_failed_tests_count=new_failed_tests_count,
+         req_anal_data=req_anal_data, app_failure_anl_count=app_failure_anl_count,
+         req_anal_perc_data=req_anal_perc_data, auto_failure_anl_count=auto_failure_anl_count,
+         new_tests_count=new_tests_count,
+         passed_test_dif=passed_test_dif,
+         failed_test_dif=failed_test_dif,
+         suite_avg_dur_data=suite_avg_dur_data,
+         test_avg_dur_data=test_avg_dur_data,
+         common_failed_suites=common_failed_suites,
+         db_name=db)
+
+    else:
+        return redirect(url_for('redirect_url'))
+
 @app.route('/<db>/dashboardRecentFive', methods=['GET'])
 def dashboardRecentFive(db):
     cursor = mysql.connection.cursor()
@@ -192,7 +270,7 @@ def dashboardRecentFive(db):
         cursor.execute("SELECT Execution_Id, Execution_Pass, Execution_Fail, Execution_Time from TB_EXECUTION order by Execution_Id desc LIMIT 5;")
         exe_id_filter_data = cursor.fetchall()
 
-        cursor.execute("SELECT Suite_Name, Suite_Fail, COUNT(Suite_Name) as F from TB_SUITE WHERE Suite_Status='FAIL' AND Execution_Id >= %s GROUP BY Suite_Name HAVING COUNT(Suite_Name) > 1 ORDER BY F DESC, Suite_Fail DESC LIMIT 5;" % exe_info[-1][0])
+        cursor.execute("SELECT b.Suite_Name, a.Suite_Fail, b.Occurence from TB_SUITE a INNER JOIN (Select Suite_Name, Count(Suite_Name) as Occurence From TB_SUITE WHERE Suite_Status = 'FAIL' AND Execution_Id>=%s GROUP BY Suite_Name HAVING COUNT(Suite_Name) > 1) b ON a.Suite_Name = b.Suite_Name WHERE Suite_Status='FAIL' AND Execution_Id=%s ORDER BY b.Occurence DESC, a.Suite_Fail DESC LIMIT 5;" % (exe_info[-1][0], exe_info[0][0]))
         common_failed_suites = cursor.fetchall()
 
         # new tests
@@ -230,7 +308,7 @@ def dashboardRecentTen(db):
         cursor.execute("SELECT Execution_Id, Execution_Pass, Execution_Fail, Execution_Time from TB_EXECUTION order by Execution_Id desc LIMIT 10;")
         exe_id_filter_data = cursor.fetchall()
 
-        cursor.execute("SELECT Suite_Name, Suite_Fail, COUNT(Suite_Name) as F from TB_SUITE WHERE Suite_Status='FAIL' AND Execution_Id >= %s GROUP BY Suite_Name HAVING COUNT(Suite_Name) > 1 ORDER BY F DESC, Suite_Fail DESC LIMIT 5;" % exe_info[-1][0])
+        cursor.execute("SELECT b.Suite_Name, a.Suite_Fail, b.Occurence from TB_SUITE a INNER JOIN (Select Suite_Name, Count(Suite_Name) as Occurence From TB_SUITE WHERE Suite_Status = 'FAIL' AND Execution_Id>=%s GROUP BY Suite_Name HAVING COUNT(Suite_Name) > 1) b ON a.Suite_Name = b.Suite_Name WHERE Suite_Status='FAIL' AND Execution_Id=%s ORDER BY b.Occurence DESC, a.Suite_Fail DESC LIMIT 5;" % (exe_info[-1][0], exe_info[0][0]))
         common_failed_suites = cursor.fetchall()
 
         # new tests
@@ -268,7 +346,7 @@ def dashboardRecentThirty(db):
         cursor.execute("SELECT Execution_Id, Execution_Pass, Execution_Fail, Execution_Time from TB_EXECUTION order by Execution_Id desc LIMIT 30;")
         exe_id_filter_data = cursor.fetchall()
 
-        cursor.execute("SELECT Suite_Name, Suite_Fail, COUNT(Suite_Name) as F from TB_SUITE WHERE Suite_Status='FAIL' AND Execution_Id >= %s GROUP BY Suite_Name HAVING COUNT(Suite_Name) > 1 ORDER BY F DESC, Suite_Fail DESC LIMIT 5;" % exe_info[-1][0])
+        cursor.execute("SELECT b.Suite_Name, a.Suite_Fail, b.Occurence from TB_SUITE a INNER JOIN (Select Suite_Name, Count(Suite_Name) as Occurence From TB_SUITE WHERE Suite_Status = 'FAIL' AND Execution_Id>=%s GROUP BY Suite_Name HAVING COUNT(Suite_Name) > 1) b ON a.Suite_Name = b.Suite_Name WHERE Suite_Status='FAIL' AND Execution_Id=%s ORDER BY b.Occurence DESC, a.Suite_Fail DESC LIMIT 5;" % (exe_info[-1][0], exe_info[0][0]))
         common_failed_suites = cursor.fetchall()
 
         # new tests
@@ -337,7 +415,7 @@ def tmetrics(db):
         textField = request.form['textField']
         rowField = request.form['rowField']
         try:
-            comment = """<b class="text-muted">Review By: </b>""" + str(userField) + """</br><b class="text-muted">Issue Type: </b>""" + str(issueField) + """</br><b class="text-muted">Comment: </b>""" + str(textField)
+            comment = """<b class="text-muted">Review By: </b>""" + str(userField) + """\n</br><b class="text-muted">Issue Type: </b>""" + str(issueField) + """\n</br><b class="text-muted">Comment: </b>""" + str(textField)
         except:
             comment = str(textField)
         cursor.execute('Update TB_TEST SET Test_Comment=\'%s\' WHERE Test_Id=%s;' % (str(comment), str(rowField)))
@@ -382,7 +460,7 @@ def eid_tmetrics(db, eid):
         textField = request.form['textField']
         rowField = request.form['rowField']
         try:
-            comment = """<b class="text-muted">Review By: </b>""" + str(userField) + """</br><b class="text-muted">Issue Type: </b>""" + str(issueField) + """</br><b class="text-muted">Comment: </b>""" + str(textField)
+            comment = """<b class="text-muted">Review By: </b>""" + str(userField) + """\n</br><b class="text-muted">Issue Type: </b>""" + str(issueField) + """\n</br><b class="text-muted">Comment: </b>""" + str(textField)
         except:
             comment = str(textField)
         cursor.execute('Update TB_TEST SET Test_Comment=\'%s\' WHERE Test_Id=%s;' % (str(comment), str(rowField)))
@@ -406,7 +484,7 @@ def eid_failures(db, eid):
         textField = request.form['textField']
         rowField = request.form['rowField']
         try:
-            comment = """<b class="text-muted">Review By: </b>""" + str(userField) + """</br><b class="text-muted">Issue Type: </b>""" + str(issueField) + """</br><b class="text-muted">Comment: </b>""" + str(textField)
+            comment = """<b class="text-muted">Review By: </b>""" + str(userField) + """\n</br><b class="text-muted">Issue Type: </b>""" + str(issueField) + """\n</br><b class="text-muted">Comment: </b>""" + str(textField)
         except:
             comment = str(textField)
         cursor.execute('Update TB_TEST SET Test_Comment=\'%s\' WHERE Test_Id=%s;' % (str(comment), str(rowField)))
@@ -430,7 +508,7 @@ def recent_failures(db):
         textField = request.form['textField']
         rowField = request.form['rowField']
         try:
-            comment = """<b class="text-muted">Review By: </b>""" + str(userField) + """</br><b class="text-muted">Issue Type: </b>""" + str(issueField) + """</br><b class="text-muted">Comment: </b>""" + str(textField)
+            comment = """<b class="text-muted">Review By: </b>""" + str(userField) + """\n</br><b class="text-muted">Issue Type: </b>""" + str(issueField) + """\n</br><b class="text-muted">Comment: </b>""" + str(textField)
         except:
             comment = str(textField)
         cursor.execute('Update TB_TEST SET Test_Comment=\'%s\' WHERE Test_Id=%s;' % (str(comment), str(rowField)))
@@ -450,7 +528,7 @@ def search(db):
         cursor = mysql.connection.cursor()
         use_db(cursor, db)
         if search:
-            cursor.execute("SELECT * from TB_TEST WHERE Test_Name LIKE '%{name}%' OR Test_Status LIKE '%{name}%' OR Execution_Id LIKE '%{name}%' ORDER BY Execution_Id DESC LIMIT 10000;".format(name=search))
+            cursor.execute("SELECT * from TB_TEST WHERE Test_Name LIKE '%{name}%' OR Test_Status LIKE '%{name}%' OR Execution_Id LIKE '%{name}%' ORDER BY Execution_Id DESC LIMIT 500;".format(name=search))
             data = cursor.fetchall()
             return render_template('search.html', data=data, db_name=db)
         else:
@@ -466,7 +544,7 @@ def flaky(db):
     last_five = cursor.fetchall()
     cursor.execute("SELECT Execution_Id from TB_EXECUTION ORDER BY Execution_Id DESC LIMIT 5;")
     last_five_ids = cursor.fetchall()
-    sql_query = "SELECT Execution_Id, Test_Name, Test_Status from TB_TEST WHERE Execution_Id >= %s ORDER BY Execution_Id DESC;" % (str(last_five[0][0]))
+    sql_query = "SELECT Test_Name, Execution_Id, Test_Status from TB_TEST WHERE Execution_Id >= %s ORDER BY Execution_Id DESC;" % (str(last_five[0][0]))
     cursor.execute(sql_query)
     data = cursor.fetchall()
     # print("==== Before Sorted Data ===")
@@ -512,6 +590,31 @@ def query(db):
     else:
         return render_template('query.html', db_name=db, error_message="")
 
+@app.route('/<db>/comment', methods=['GET', 'POST'])
+def comment(db):
+    cursor = mysql.connection.cursor()
+    use_db(cursor, db)
+    cursor.execute("SELECT Execution_Id from TB_EXECUTION order by Execution_Id desc LIMIT 1;")
+    recent_eid = cursor.fetchone()
+
+    if request.method == "POST":
+        error = request.form['error']
+        eid = request.form['eid']
+        issue = request.form['issue']
+        user = request.form['user']
+        commentMsg = request.form['comment']
+        comment = """<b class="text-muted">Review By: </b>""" + str(user) + """\n</br><b class="text-muted">Issue Type: </b>""" + str(issue) + """\n</br><b class="text-muted">Comment: </b>""" + str(commentMsg)
+
+        try:
+            cursor.execute('Update TB_TEST SET Test_Comment=\'{}\' WHERE Execution_Id={} AND Test_Error LIKE \'%{}%\';'.format(comment, eid, error))
+            mysql.connection.commit()
+            return render_template('comment.html', error_message="", recent_eid=recent_eid)
+        except Exception as e:
+            print(str(e))
+            return render_template('comment.html', error_message=str(e), recent_eid=recent_eid)
+    else:
+        return render_template('comment.html', error_message="", recent_eid=recent_eid)
+
 def use_db(cursor, db_name):
     cursor.execute("USE %s;" % db_name)
 
@@ -519,9 +622,9 @@ def sort_tests(data_list):
     out = {}
     for elem in data_list:
         try:
-            out[elem[1]].extend(elem[2:])
+            out[elem[0]].extend(elem[1:])
         except KeyError:
-            out[elem[1]] = list(elem)
+            out[elem[0]] = list(elem)
     return [tuple(values) for values in out.values()]
 
 def get_count_by_perc(data_list, max, min):
